
#include <windows.h>  
#include "combrgn.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "CombineRgn()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadIcon(wcex.hInstance, "SMALL");
			
   return RegisterClassEx( &wcex );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )
   {
      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        {
                           POINT    ptArray[3];
                           HRGN     Rgn1, Rgn2, Rgn3, Rgn4;
                           LOGBRUSH lb;
                           HBRUSH   hBrush;
                           HDC      hDC = GetDC( hWnd );

                           // Create a rectangular region.
                           //.............................
                           Rgn1 = CreateRectRgn( 10, 10, 110, 110 );

                           // Create an elliptic region.
                           //...........................
                           Rgn2 = CreateEllipticRgn( 60, 60, 160, 160 );

                           // Create a polygonal region consisting of three points.
                           //......................................................
                           ptArray[0].x = 10;  ptArray[0].y = 160;
                           ptArray[1].x = 60;  ptArray[1].y = 60;
                           ptArray[2].x = 110; ptArray[2].y = 160;
                           Rgn3 = CreatePolygonRgn( ptArray, 
                                                    sizeof(ptArray)/sizeof(POINT), ALTERNATE );


                           // Create a dummy region to combine into.
                           //.......................................
                           Rgn4 = CreateRectRgn( 0, 0, 10, 10 ); 

                           // Combine the elliptic and rectangular regions. 
                           //..............................................
                           CombineRgn( Rgn4, Rgn2, Rgn1, RGN_OR );

                           // Combine the results with the polygon region.
                           //.............................................
                           CombineRgn( Rgn4, Rgn4, Rgn3, RGN_OR );

                           // Now create a hatched brush and draw the resulting region.
                           //..........................................................
                           lb.lbStyle = BS_HATCHED;
                           lb.lbColor = RGB(0, 0, 0);
                           lb.lbHatch = HS_DIAGCROSS;
                           hBrush = CreateBrushIndirect( &lb );

                           FillRgn( hDC, Rgn4, hBrush );

                           //  Finally, delete all created GDI objects.
                           //..........................................
                           DeleteObject( hBrush );
                           DeleteObject( Rgn1 );
                           DeleteObject( Rgn2 );
                           DeleteObject( Rgn3 );
                           DeleteObject( Rgn4 );

                           ReleaseDC( hWnd, hDC );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
